//////////////////////////////////////////////////////////////
/////           Divers schemas de volumes finis        ///////
/////               pour pbs elliptiques 1D            ///////
/////                                                  ///////
/////                 Programme principal              ///////
/////       permettant le calcul et le trace des sol.  ///////
/////                                                  ///////
/////                F. Boyer  &  S. Krell             ///////
/////                     Mars 2010                    ///////
//////////////////////////////////////////////////////////////

//**************************************************************************
// On efface toutes les variables et on charge les fonctions utiles
//**************************************************************************
clear
clc
exec('maillages1D.sci');
exec('schemas1D.sci');
exec('donnees1D.sci');


//**************************************************************************
// Saisie par l'utilisateur des données du calcul
//**************************************************************************

// choix du cas test

printf('========================================\n');
printf('Choix du cas test :\n');
for i=1:length(cas_test)
  printf('%d) %s\n',i,cas_test(i).nom);
end;

choix=-1;
while (int(choix)<=0) | (int(choix)>length(cas_test))
  choix=input("Faites votre choix :");
end;
choix=int(choix);

// Chargement des donnees

donnees=cas_test(choix);

// choix du problème à résoudre 


  printf('========================================\n');
  printf('Choix du problème :\n');
  printf('1) Laplacien Dirichlet homogène\n');
  printf('2) Laplacien Dirichlet non-homogène\n');
  printf('3) Diffusion générale Dirichlet non-homogène\n');
  printf('4) Diffusion générale Neumann non-homogène\n');
  choix_pb=-1;
  while (int(choix_pb)<=0) | (int(choix_pb)>4)
    choix_pb=input("Faites votre choix :");
  end;
  choix_pb=int(choix_pb);

// si le coeff de diffusion est variable : choix de la methode

if isfield(donnees,"methode") then
  printf('========================================\n');
  printf('Choix de la méthode de calcul du coeff de diffusion :\n');
  printf('1) exacte\n');
  printf('2) moyenne arithmetique\n');
  printf('3) moyenne harmonique\n');

  choix=-1;
  while (int(choix)<=0) | (int(choix)>3)
    choix=input("Faites votre choix :");
  end;
  
  select choix
   case 1
    donnees.methode="exacte";
   case 2
    donnees.methode="arithmetique";
   case 3
    donnees.methode="harmonique";
  end;
end;

// choix du type de maillage

printf('========================================\n');
printf('Choix du maillage :\n');
printf('1) uniforme\n');
printf('2) alterne\n');
printf('3) aleatoire\n');
printf('4) stretch\n');
choix=-1;
while (int(choix)<=0) | (int(choix)>4)
  choix=input("Faites votre choix :");
end;
choix=int(choix);

// choix du nombre de mailles

printf('========================================\n');
N=input('Nombre de mailles :');
N=int(N);

// Creation du maillage

select choix
 case 1 then  [maillage]=maillage_uniforme(N);
 case 2 then  [maillage]=maillage_alterne(N);
 case 3 then  [maillage]=maillage_aleatoire(N);
 case 4 then  [maillage]=maillage_stretch(N);
end;

//**************************************************************************
// Construction du schéma et résolution du système linéaire
//**************************************************************************

// Construction de la matrice et du second membre

select choix_pb
  case 1
     [A,b]=vf_laplacien_dirh(maillage,donnees);
  case 2
     [A,b]=vf_laplacien_dirnh(maillage,donnees);
  case 3
     [A,b]=vf_diffusion_dirnh(maillage,donnees);
  case 4
     [A,b]=vf_diffusion_neunh(maillage,donnees);
end;


printf('Resolution\n');
try
  // On essaie d'abord UMFPACK
  sol=umfpack(A,'\',b);
catch
  // Sinon on résout par le solveur Scilab
  sol=A\b;
end,

//**************************************************************************
// Affichage et calcul de l'erreur si disponible
//**************************************************************************


  valeur_a_gauche=donnees.bordD(0);
  valeur_a_droite=donnees.bordD(1);


  scf(0);
  clf;
  plot([0; maillage.centres; 1],[valeur_a_gauche; sol; valeur_a_droite],'b');
  
  
if isfield(donnees,'uexacte') then

  solexacte=feval(maillage.centres,donnees.uexacte);
  printf('========================================\n');
  printf('Erreur en norme L2 = %e\n',normeL2(maillage,sol-solexacte));
  printf('Erreur en norme Linf = %e\n',max(abs(sol-solexacte)));
  printf('Erreur en norme H1 = %e\n',normeH1(maillage,sol-solexacte));

  plot([0; maillage.centres; 1],[valeur_a_gauche; solexacte; valeur_a_droite],'r');
  
  xtitle([donnees.nom; maillage.nom+', nb_vol = '+string(maillage.nb_vol)]);
  legend('Solution Approchee','Solution Exacte');
  
else
  
    xtitle('Solution Approchee');
    legend('Solution Approchee');

end;

printf('==============  FIN  ===================\n');
  
//**************************************************************************
//                                    FIN
//**************************************************************************