//////////////////////////////////////////////////////////////
/////           Divers schemas de volumes finis        ///////
/////               pour pbs elliptiques 1D            ///////
/////                                                  ///////
/////                 Programme principal              ///////
/////       permettant le trace de courbes d'erreurs   ///////
/////                                                  ///////
/////                F. Boyer  &  S. Krell             ///////
/////                     Mars 2010                    ///////
//////////////////////////////////////////////////////////////

//**************************************************************************
// On efface toutes les variables et on charge les fonctions utiles
//**************************************************************************
clear
clc
exec('maillages1D.sci');
exec('schemas1D.sci');
exec('donnees1D.sci');

//**************************************************************************
// Saisie par l'utilisateur des données du calcul
//**************************************************************************

// choix du cas test

printf('========================================\n');
printf('Choix du cas test :\n');
for i=1:length(cas_test)
  printf('%d) %s\n',i,cas_test(i).nom);
end;

choix=-1;
while (int(choix)<=0) | (int(choix)>length(cas_test))
  choix=input("Faites votre choix :");
end;
choix=int(choix);

// Chargement des donnees

donnees=cas_test(choix);


if ~isfield(donnees,"uexacte") then
  printf("La solution exacte n''est pas definie. "+...
	 "Impossible de calculer des erreurs\n");
  halt;
end;

// choix du problème à résoudre 


  printf('========================================\n');
  printf('Choix du problème :\n');
  printf('1) Laplacien Dirichlet homogène\n');
  printf('2) Laplacien Dirichlet non-homogène\n');
  printf('3) Diffusion générale Dirichlet non-homogène\n');
  printf('4) Diffusion générale Neumann non-homogène\n');
  choix_pb=-1;
  while (int(choix_pb)<=0) | (int(choix_pb)>4)
    choix_pb=input("Faites votre choix :");
  end;
  choix_pb=int(choix_pb);
  
// si le coeff de diffusion est variable : choix de la methode

if isfield(donnees,"methode") then
   printf('========================================\n');
   printf('Choix de la méthode de calcul du coeff de diffusion :\n');
   printf('1) exacte\n');
   printf('2) moyenne arithmetique\n');
   printf('3) moyenne harmonique\n');

   choix=-1;
   while (int(choix)<=0) | (int(choix)>3)
     choix=input("Faites votre choix :");
   end;
   
   select choix
    case 1
     donnees.methode="exacte";
    case 2
     donnees.methode="arithmetique";
    case 3
     donnees.methode="harmonique";
   end;
end;

// choix du type de maillage

printf('========================================\n');
printf('Choix du maillage :\n');
printf('1) uniforme\n');
printf('2) alterne\n');
printf('3) aleatoire\n');
printf('4) stretch\n');
choix=-1;
while (int(choix)<=0) | (int(choix)>4)
  choix=input("Faites votre choix :");
end;
choix=int(choix);

// choix des différentes tailles de maillage pour le trace des courbes

printf('========================================\n');
nb_dep=input('Nombre de mailles du maillage le plus grossier:');
nb_dep=int(nb_dep);

nb_int=input('Pas:');
nb_int=int(nb_int);

nb_fin=input('Nombre de mailles du maillage le plus fin:');
nb_fin=int(nb_fin);

NN=[nb_dep:nb_int:nb_fin];//linspace(nb_dep,nb_fin,nb_int);//[nb_dep:nb_int:nb_fin];
errinf=zeros(1,length(NN));
errL2=zeros(1,length(NN));
errH1=zeros(1,length(NN));
pas=zeros(1,length(NN));

printf('========================================\n');

for i=1:length(NN)     // boucle sur les différents maillages
  
  // Creation du maillage

  select choix
    case 1 then  [maillage]=maillage_uniforme(NN(i));
    case 2 then  [maillage]=maillage_alterne(NN(i));
    case 3 then  [maillage]=maillage_aleatoire(NN(i));
    case 4 then  [maillage]=maillage_stretch(NN(i));
  end;
  
  // Calcul de la solution exacte aux centres des volumes de contrôle
  
  solexacte=feval(maillage.centres,donnees.uexacte);
  
  //**************************************************************************
  // Construction du schéma et résolution du système linéaire
  //**************************************************************************

  
// Construction de la matrice et du second membre

  select choix_pb
   case 1
    [A,b]=vf_laplacien_dirh(maillage,donnees);
   case 2
    [A,b]=vf_laplacien_dirnh(maillage,donnees);
   case 3
    [A,b]=vf_diffusion_dirnh(maillage,donnees);
   case 4
    [A,b]=vf_diffusion_neunh(maillage,donnees);
  end;

  printf('Resolution\n');
  try
      // On essaie d'abord UMFPACK
    sol=umfpack(A,'\',b);
  catch
      // Sinon on résout par le solveur Scilab
    sol=A\b;
  end;
  
  // Calcul des erreurs et de la taille du pas du maillage
  
  errinf(i)=max(abs(sol-solexacte));
  errL2(i)=normeL2(maillage,sol-solexacte);
  errH1(i)=normeH1(maillage,sol-solexacte);
  pas(i)=max(maillage.mes);
  
end


//**************************************************************************
//     Tracé des courbes d'erreur
//**************************************************************************


scf(0);
clf;

// Erreur en norme infinie

subplot(1,3,1)
c1=regress(log(pas),log(errinf));
xtitle([ donnees.nom; 
	 maillage.nom; ...
	 'Ordre de convergence norme infinie = "+string(c1(2))]);
plot(pas,errinf,'-+b');
a=gca();
a.log_flags="ll";

// Erreur en norme L2

subplot(1,3,2)
c2=regress(log(pas),log(errL2));
xtitle([ donnees.nom; 
	 maillage.nom; ...
	 'Ordre de convergence norme L2 = "+string(c2(2))]);
plot(pas,errL2,'-+b');
a=gca();
a.log_flags="ll";

// Erreur en norme H1

subplot(1,3,3)
c3=regress(log(pas),log(errH1));
xtitle([ donnees.nom; 
	 maillage.nom; ...
	 'Ordre de convergence norme H1 = "+string(c3(2))]);
plot(pas,errH1,'-+b');
a=gca();
a.log_flags="ll";

printf('========================================\n');
printf('Ordre de convergence en norme infinie : %g\n',c1(2));
printf('Ordre de convergence en norme L2 : %g\n',c2(2));
printf('Ordre de convergence en norme H1 : %g\n',c3(2));
printf('==============  FIN  ===================\n');


//**************************************************************************
//           FIN
//**************************************************************************