//////////////////////////////////////////////////////////////
/////           Divers schemas de volumes finis        ///////
/////               pour pbs elliptiques 1D            ///////
/////                                                  ///////
/////                 Jeux de données                  ///////
/////                                                  ///////
/////                F. Boyer  &  S. Krell             ///////
/////                     Mars 2010                    ///////
//////////////////////////////////////////////////////////////


//**************************************************************************
//    Le format des jeux de données
//**************************************************************************
//  Chaque jeu de données contient (ou pas !) un certain nombre d'éléments
//  *  nom :  chaîne de caractère qui décrit le cas test
//  *  source    : fonction x->f(x) qui décrit le terme source
//  *  uexacte   : fonction x->ue(x) qui donne la solution exacte, si elle existe
//  *  coeff_k   : fonction x->k(x) qui décrit le coefficient de diffusion
//  *  bordD     : fonction x->borD(x) qui renvoie les données de Dirichlet au bord
//  *  bordN : fonction qui renvoie la donnee de Neumann au bord
//  *  dirac     : position dans l'intervalle ]0,1[ d'un terme de Dirac en second membre
//  *  moyenne   : valeur de la moyenne de la solution (utile pour Neumann)
//  *  methode   : choix de la methode de calcul du coefficient de diffusion 
//                 à l'interface entre deux volumes de contrôle
////////////////////////////////////////////////////////////////////////////

function y=coeff_cte(x)
  y=ones(x);
endfunction

cas_test=list();

//*********                     un polynome de degré 2 nul au bord
//*********                     k(x)=1

function y=ue1(x)
  y=1/(2)*x.*(1-x);  
endfunction

function y=flux_ue1(x)
  y=-1/(2);  
endfunction

function y=f1(x)
  y=ones(1,length(x));  
endfunction

cas_test($+1)=struct("nom","u(x)=x(1-x)/2, k(x)=1",...
		     "uexacte",ue1,...
		     "bordD",ue1,...
		     "source",f1,...
		     "moyenne",1/12,...
		     "bordN",flux_ue1,...
		     "coeff_k",coeff_cte);

//*********                    u(x)=sin(pi x)
//*********                    k(x)=1


function y=ue2(x)
  y=sin(%pi.*x);  
endfunction

function y=flux_ue2(x)
  y=-%pi;  
endfunction

function y=f2(x)
  y=%pi*%pi.*sin(%pi.*x);  
endfunction

cas_test($+1)=struct("nom","u(x)=sin(pi*x), k(x)=1",...
		     "uexacte",ue2,...
		     "bordD",ue2,...
		     "source",f2,...
		     "moyenne",2/%pi,...
		     "bordN",flux_ue2,...
		     "coeff_k",coeff_cte);

//*********           u(x)=sin(pi x)
//*********           k(x)=1


function y=ue2_bis(x)
  y=cos(%pi.*x);  
endfunction

function y=flux_ue2_bis(x)
  y=0;  
endfunction

function y=f2_bis(x)
  y=%pi*%pi.*cos(%pi.*x);  
endfunction

cas_test($+1)=struct("nom","u(x)=cos(pi*x), k(x)=1",...
		     "uexacte",ue2_bis,...
		     "bordD",ue2_bis,...
		     "source",f2_bis,...
		     "moyenne",0,...
		     "bordN",flux_ue2_bis,...
		     "coeff_k",coeff_cte);



//*********                       u(x)=tanh((x-0.3)/eps)
//*********                       k(x)=1


param3=0.1;

function y=ue3(x)
  y=tanh((x-0.3)/param3); 
endfunction

function y=flux_ue3(x)
  y=-(1/param3)*(1-tanh((x-0.3)/param3)^2)*((x-0.5)/abs(x-0.5)); 
endfunction

function y=f3(x)
  y=2/param3^2*tanh((x-0.3)/param3)*(1-(tanh((x-0.3)/param3)^2));  
endfunction

cas_test($+1)=struct("nom","u(x)=tanh((x-0.3)/eps), eps='+string(param3)+",k(x)=1",...
		     "uexacte",ue3,...
		     "bordD",ue3,...
		     "source",f3,...
		     "moyenne",param3*(log(cosh((1-0.3)/param3))-log(cosh(-0.3/param3))),...
		     "bordN",flux_ue3,...
		     "coeff_k",coeff_cte);


//*********                       u(x)=sin(pi x)
//*********                       k(x)=(1+1/2*cos(pi x))
//*********              par defaut le coefficient k_{i+1/2} est évalué
//*********                                             de façon exacte


function y=ue4(x)
  y=sin(%pi.*x);  
endfunction

function y=k4(x)
  y=1+0.5*cos(%pi.*x);  
endfunction

function y=f4(x)
  y=%pi*%pi.*sin(%pi.*x).*(1+cos(%pi.*x));  
endfunction

function y=flux_ue4(x)
  y=-%pi*k4(x);  
endfunction


cas_test($+1)=struct("nom","u(x)=sin(pi*x), k(x)=1+0.5*cos(pi*x)",...
		     "uexacte",ue4,...
		     "bordD",ue4,...
		     "source",f4,...
		     "moyenne",2/%pi,...
		     "bordN",flux_ue4,...
		     "coeff_k",k4,...
		     "methode","exacte");



//*********                       u(x)=une fonction régulière par morceaux
//*********                       k(x)=constant par morceaux
//*********              par defaut le coefficient k_{i+1/2} est évalué
//*********                                             de façon exacte


function y=fdis(x)
  y=(x<0.5).*(-2*kdis(0)*(20.*(x-0.5).^3+12*(x-0.5).^2+6*(x-0.5)+2) ...
      -kdis(0)*(x-0.5).*(60.*(x-0.5).^2+24*(x-0.5)+6)) ...
      +(x>=0.5).*(-2*kdis(1)*(kdis(0)*%pi*cos(%pi.*x)-5*%pi*sin(%pi.*x) ) ...
      +kdis(1)*(x-0.5).*(kdis(0)*%pi*%pi*sin(%pi.*x)+5*%pi*%pi*cos(%pi.*x) )  );
endfunction

function y=udis(x)
  y=   (x<0.5).*((x-0.5).*(5.*(x-0.5).^4+4*(x-0.5).^3+3*(x-0.5).^2+2*(x-0.5)+kdis(1))) ...
      +(x>=0.5).*((x-0.5).*(kdis(0)*sin(%pi.*x)+5*cos(%pi.*x) ));
endfunction

k_gauche=1;
k_droit=10;

function y=kdis(x)
  y=(x<0.5).*(k_gauche) + (x>=0.5).*(k_droit);
endfunction

cas_test($+1)=struct("nom","u=reguliere par morceaux, k="+string(k_gauche)+" pour x<1/2 et k="+string(k_droit)+" pour x>1/2",...
		     "uexacte",udis,...
		     "bordD",udis,...
		     "source",fdis,...
		     "coeff_k",kdis,...
		     "methode","exacte");


//*********            u(x)= affine par morceaux
//*********            k(x)=1
//*********            Terme de dirac 

x_dirac=0.235;

function y=ue_aff_pm(x)
  y=-0.5*abs(x-x_dirac);
endfunction

function y=f_nulle(x)
  y=0;
endfunction

cas_test($+1)=struct("nom","u affine par morceaux. Dirac en x='+string(x_dirac)+', k(x)=1",...
		     "uexacte",ue_aff_pm,...
		     "source",f_nulle,...
		     "bordD",ue_aff_pm,...
		     "coeff_k",coeff_cte,...
		     "dirac",x_dirac);


//*********                       u(x)=abs(sin(2 pi x))
//*********                       k(x)=1
//*********                       Terme de dirac en x=0.5

function y=ue5(x)
  y=-sin(2*%pi*pmodulo(x,0.5))/(4*%pi);
endfunction

function y=flux_ue5(x)
  y=1/2;
endfunction

function y=f5(x)
  y=-%pi*sin(2*%pi*pmodulo(x,0.5));
endfunction

cas_test($+1)=struct("nom","u =|sin|, Terme source avec Dirac en 1/2, k(x)=1",...
		     "uexacte",ue5,...
		     "source",f5,...
		     "bordD",ue5,...
		     "coeff_k",coeff_cte,...
		     "moyenne",-1/(2*%pi^2),...
		     "bordN",flux_ue5,...
		     "dirac",0.5);

