//////////////////////////////////////////////////////////////
/////           Divers schemas de volumes finis        ///////
/////               pour pbs elliptiques 2D            ///////
/////                                                  ///////
/////                 Programme principal              ///////
/////       permettant le trace de courbes d'erreurs   ///////
/////                                                  ///////
/////                F. Boyer  &  S. Krell             ///////
/////                     Mars 2010                    ///////
//////////////////////////////////////////////////////////////

//**************************************************************************
// On efface toutes les variables et on charge les fonctions utiles
//**************************************************************************
clear
clc
stacksize(10000000);
exec('maillages2D.sci');
exec('schemas2D.sci');
exec('donnees2D.sci');

//**************************************************************************
// Saisie par l'utilisateur des données du calcul
//**************************************************************************

// choix du cas test

l=list();
for i=1:length(cas_test)
  if isfield(cas_test(i),'uexacte') then
    l($+1)=cas_test(i);
  end;
end
cas_test=l;

printf('========================================\n');
printf('Choix du cas test :\n');
for i=1:length(cas_test)
  printf('%d) %s\n',i,cas_test(i).nom);
end;

choix=-1;
while (int(choix)<=0) | (int(choix)>length(cas_test))
  choix=input("Faites votre choix :");
end;
choix=int(choix);

// Chargement des donnees

donnees=cas_test(choix);

// On regarde si des maillages sont imposés dans le jeu de données

if isfield(donnees,"maillages") then
  l=list();
  for a=donnees.maillages,
    for b=liste_maillages,
      if a==b.nom then 
	l($+1)=b;
      end;
    end;
  end;
  liste_maillages=l;
end;

// On ne garde que les maillages qui ont plusieurs niveaux de raffinement

l=list();
for i=1:length(liste_maillages)
  if isfield(liste_maillages(i),'indice_min') then
    l($+1)=liste_maillages(i);
  end;
end
liste_maillages=l;

// On fait un choix parmi les possibilités restantes

printf('========================================\n');
printf('Choix du maillage :\n');
for i=1:length(liste_maillages)
  printf('%d) %s\n',i,liste_maillages(i).nom);
end;

choix=-1;
while (int(choix)<=0) | (int(choix)>length(liste_maillages))
  choix=input("Faites votre choix :");
end;

choix=int(choix);
maillage=liste_maillages(choix);


// choix des différentes tailles de maillage pour le trace des courbes
if isfield(maillage,"indice_min") then
  printf('========================================\n');
  printf('Niveaux de raffinement utilises entre %d et %d :\n',maillage.indice_min,maillage.indice_max);
  nb_dep=-1;
  while (int(nb_dep)<maillage.indice_min) | (int(nb_dep)>maillage.indice_max)
    nb_dep=input("Choix du niveau le plus grosier :");
  end;
  nb_fin=-1;
  while (int(nb_fin)<nb_dep) | (int(nb_fin)>maillage.indice_max)
    nb_fin=input("Choix du niveau le plus fin :");
  end;
end
// si le coeff de diffusion est variable : choix de la methode

if isfield(donnees,"methode") then
  printf('========================================\n');
  printf('Choix de la méthode de calcul du coeff de diffusion :\n');
  printf('1) exacte\n');
  printf('2) moyenne arithmetique\n');
  printf('3) moyenne harmonique\n');

  choix=-1;
  while (int(choix)<=0) | (int(choix)>3)
    choix=input("Faites votre choix :");
  end;
  
  select choix
   case 1
    donnees.methode="exacte";
   case 2
    donnees.methode="arithmetique";
   case 3
    donnees.methode="harmonique";
  end;
end;
 
NN=nb_fin-nb_dep+1;

errinf=zeros(1,length(NN));
errL2=zeros(1,length(NN));
errH1=zeros(1,length(NN));
pas=zeros(1,length(NN));

// choix du schéma : VF4 ou DDFV
if isfield(donnees,"coeff_mat") then
  choix=3;
  printf('========================================\n');
  printf('Schéma DDFV\n');
else
  printf('========================================\n');
  printf('Choix du schéma à utiliser :\n');
  printf('1) VF4\n');
  printf('2) DDFV\n');
  printf('3) DDFV hétérogène anisotrope\n');
  choix=-1;
  while ((int(choix)<1) | (int(choix)>3))
    choix=input("Faites votre choix :");
  end;
end;
printf('========================================\n');

for i=nb_dep:nb_fin     // boucle sur les différents maillages
  
  // Chargement du maillage
  nom_maillage=maillage.nom+'_'+string(i);
  m = lecture_maillage(rep_maillages+nom_maillage);
  
  printf('Calcul sur le maillage '+nom_maillage+'\n');
  
  // Calcul de la solution exacte aux centres des volumes de contrôle
  if isfield(donnees,"uexacte") then
   solexacte=eval_fonction(m.centres,donnees.uexacte);
  end;
  
  //**************************************************************************
  // Construction du schéma et résolution du système linéaire
  //**************************************************************************

  // Construction de la matrice et du second membre

  select choix
   case 1
     [A,b]=const_schema_VF4(m,donnees);
   case 2
     [A,b]=const_schema_DDFV(m,donnees);
   case 3
    [A,b]=const_schema_DDFV_mat(m,donnees);
  end;

  printf('Resolution\n');
  try
      // On essaie d'abord UMFPACK
    sol=umfpack(A,'\',b);
  catch
      // Sinon on résout par le solveur Scilab
    sol=A\b;
end;

  // Calcul des erreurs et de la taille du pas du maillage
  
  errinf(i+1-nb_dep)=max(abs(sol(1:m.nb_vol)-solexacte));
  errL2(i+1-nb_dep)=norme_L2(m,sol(1:m.nb_vol)-solexacte);
  errH1(i+1-nb_dep)=norme_H1(m,sol(1:m.nb_vol)-solexacte);
  pas(i+1-nb_dep)=max(m.aretes(:,_MES));
  
end


//**************************************************************************
//     Tracé des courbes d'erreur
//**************************************************************************


scf(0);
clf;


// Erreur en norme infinie

subplot(1,3,1)
c1=regress(log(pas),log(errinf));
xtitle([ donnees.nom; 
	 m.nom; ...
	 'Ordre de convergence norme infinie = "+string(c1(2))]);
plot(pas,errinf,'-+b');
a=gca();
a.log_flags="ll";

// Erreur en norme L2

subplot(1,3,2)
c2=regress(log(pas),log(errL2));
xtitle([ donnees.nom; 
	 m.nom; ...
	 'Ordre de convergence norme L2 = "+string(c2(2))]);
plot(pas,errL2,'-+b');
a=gca();
a.log_flags="ll";

// Erreur en norme H1

subplot(1,3,3)
c3=regress(log(pas),log(errH1));
xtitle([ donnees.nom; 
	 maillage.nom; ...
	 'Ordre de convergence norme H1 = "+string(c3(2))]);
plot(pas,errH1,'-+b');
a=gca();
a.log_flags="ll";

printf('========================================\n');
printf('Ordre de convergence en norme infinie : %g\n',c1(2));
printf('Ordre de convergence en norme L2 : %g\n',c2(2));
printf('Ordre de convergence en norme H1 : %g\n',c3(2));
printf('==============  FIN  ===================\n');

//**************************************************************************
//           FIN
//**************************************************************************